@@ -726,7 +726,6 @@ int crush_bucket_add_item(struct crush_bucket *b, int item, int weight)
 	default:
 		return -1;
 	}
-	return 0;
 }
 
 /************************************************/
@@ -746,10 +745,7 @@ int crush_remove_uniform_bucket_item(struct crush_bucket_uniform *bucket, int it
 	for (j = i; j < bucket->h.size; j++)
 		bucket->h.items[j] = bucket->h.items[j+1];
 	newsize = --bucket->h.size;
-	if (bucket->item_weight < bucket->h.weight)
-		bucket->h.weight -= bucket->item_weight;
-	else
-		bucket->h.weight = 0;
+	bucket->h.weight -= bucket->item_weight;
 
 	if ((_realloc = realloc(bucket->h.items, sizeof(__s32)*newsize)) == NULL) {
 		return -ENOMEM;
@@ -782,10 +778,7 @@ int crush_remove_list_bucket_item(struct crush_bucket_list *bucket, int item)
 		bucket->item_weights[j] = bucket->item_weights[j+1];
 		bucket->sum_weights[j] = bucket->sum_weights[j+1] - weight;
 	}
-	if (weight < bucket->h.weight)
-		bucket->h.weight -= weight;
-	else
-		bucket->h.weight = 0;
+	bucket->h.weight -= weight;
 	newsize = --bucket->h.size;
 	
 	void *_realloc = NULL;
@@ -836,10 +829,7 @@ int crush_remove_tree_bucket_item(struct crush_bucket_tree *bucket, int item)
 			bucket->node_weights[node] -= weight;
 			printf(" node %d weight %d\n", node, bucket->node_weights[node]);
 		}
-		if (weight < bucket->h.weight)
-			bucket->h.weight -= weight;
-		else
-			bucket->h.weight = 0;
+		bucket->h.weight -= weight;
 		break;
 	}
 	if (i == bucket->h.size)
@@ -894,10 +884,7 @@ int crush_remove_straw_bucket_item(struct crush_bucket_straw *bucket, int item)
 	for (i = 0; i < bucket->h.size; i++) {
 		if (bucket->h.items[i] == item) {
 			bucket->h.size--;
-			if (bucket->item_weights[i] < bucket->h.weight)
-				bucket->h.weight -= bucket->item_weights[i];
-			else
-				bucket->h.weight = 0;
+			bucket->h.weight -= bucket->item_weights[i];
 			for (j = i; j < bucket->h.size; j++) {
 				bucket->h.items[j] = bucket->h.items[j+1];
 				bucket->item_weights[j] = bucket->item_weights[j+1];
@@ -951,7 +938,6 @@ int crush_bucket_remove_item(struct crush_bucket *b, int item)
 	default:
 		return -1;
 	}
-	return 0;
 }
 
 
@@ -1057,7 +1043,6 @@ int crush_bucket_adjust_item_weight(struct crush_bucket *b, int item, int weight
 	default:
 		return -1;
 	}
-	return 0;
 }
 
 /************************************************/
@@ -1173,7 +1158,6 @@ int crush_reweight_bucket(struct crush_map *crush, struct crush_bucket *b)
 	default:
 		return -1;
 	}
-	return 0;
 }
 
 /***************************/
