@@ -709,6 +709,19 @@ int rados_pool_create_with_crush_rule(rados_t cluster, const char *pool_name,
 int rados_pool_create_with_all(rados_t cluster, const char *pool_name, uint64_t auid,
 			       uint8_t crush_rule_num);
 
+/**
+ * Returns the pool that is the base tier for this pool.
+ *
+ * The return value is the ID of the pool that should be used to read from/write to.
+ * If tiering is not set up for the pool, returns \c pool.
+ *
+ * @param cluster the cluster the pool is in
+ * @param pool ID of the pool to query
+ * @param[out] base_tier base tier, or \c pool if tiering is not configured
+ * @returns 0 on success, negative error code on failure
+ */
+int rados_pool_get_base_tier(rados_t cluster, int64_t pool, int64_t* base_tier);
+
 /**
  * Delete a pool and all data inside it
  *
@@ -1070,7 +1083,8 @@ int rados_ioctx_snap_get_stamp(rados_ioctx_t io, rados_snap_t id, time_t *t);
 uint64_t rados_get_last_version(rados_ioctx_t io);
 
 /**
- * Write data to an object
+ * Write *len* bytes from *buf* into the *oid* object, starting at
+ * offset *off*. The value of *len* must be <= UINT_MAX/2.
  *
  * @note This will never return a positive value not equal to len.
  * @param io the io context in which the write will occur
@@ -1083,7 +1097,8 @@ uint64_t rados_get_last_version(rados_ioctx_t io);
 int rados_write(rados_ioctx_t io, const char *oid, const char *buf, size_t len, uint64_t off);
 
 /**
- * Write an entire object
+ * Write *len* bytes from *buf* into the *oid* object. The value of
+ * *len* must be <= UINT_MAX/2.
  *
  * The object is filled with the provided data. If the object exists,
  * it is atomically truncated and then written.
@@ -1118,7 +1133,8 @@ int rados_clone_range(rados_ioctx_t io, const char *dst, uint64_t dst_off,
                       const char *src, uint64_t src_off, size_t len);
 
 /**
- * Append data to an object
+ * Append *len* bytes from *buf* into the *oid* object. The value of
+ * *len* must be <= UINT_MAX/2.
  *
  * @param io the context to operate in
  * @param oid the name of the object
@@ -1863,7 +1879,7 @@ int rados_set_alloc_hint(rados_ioctx_t io, const char *o,
  *
  * @returns non-NULL on success, NULL on memory allocation error.
  */
-rados_write_op_t rados_create_write_op();
+rados_write_op_t rados_create_write_op(void);
 
 /**
  * Free a rados_write_op_t, must be called when you're done with it.
@@ -2104,7 +2120,7 @@ int rados_aio_write_op_operate(rados_write_op_t write_op,
  *
  * @returns non-NULL on success, NULL on memory allocation error.
  */
-rados_read_op_t rados_create_read_op();
+rados_read_op_t rados_create_read_op(void);
 
 /**
  * Free a rados_read_op_t, must be called when you're done with it.
