@@ -47,6 +47,9 @@ enum {
   l_librbd_notify,
   l_librbd_resize,
 
+  l_librbd_readahead,
+  l_librbd_readahead_bytes,
+
   l_librbd_last,
 };
 
@@ -114,6 +117,7 @@ namespace librbd {
 			bool *is_protected);
   int add_snap(ImageCtx *ictx, const char *snap_name);
   int rm_snap(ImageCtx *ictx, const char *snap_name);
+  int refresh_parent(ImageCtx *ictx);
   int ictx_check(ImageCtx *ictx);
   int ictx_refresh(ImageCtx *ictx);
   int copy(ImageCtx *ictx, IoCtx& dest_md_ctx, const char *destname,
