@@ -9,12 +9,11 @@
 #include <ostream>
 #include <set>
 #include <map>
-using namespace std;
 
 #include "common/config.h"
 #include "common/Clock.h"
 #include "common/DecayCounter.h"
-#include "include/Context.h"
+#include "MDSContext.h"
 
 #include "include/frag.h"
 #include "include/xlist.h"
@@ -25,6 +24,8 @@ using namespace std;
 #include <boost/pool/pool.hpp>
 #include "include/assert.h"
 #include "include/hash_namespace.h"
+#include <boost/serialization/strong_typedef.hpp>
+
 
 #define CEPH_FS_ONDISK_MAGIC "ceph fs volume v011"
 
@@ -70,6 +71,12 @@ using namespace std;
 #define MDS_TRAVERSE_DISCOVERXLOCK 3    // succeeds on (foreign?) null, xlocked dentries.
 
 
+BOOST_STRONG_TYPEDEF(int32_t, mds_rank_t)
+BOOST_STRONG_TYPEDEF(uint64_t, mds_gid_t)
+extern const mds_gid_t MDS_GID_NONE;
+extern const mds_rank_t MDS_RANK_NONE;
+
+
 extern long g_num_ino, g_num_dir, g_num_dn, g_num_cap;
 extern long g_num_inoa, g_num_dira, g_num_dna, g_num_capa;
 extern long g_num_inos, g_num_dirs, g_num_dns, g_num_caps;
@@ -161,7 +168,7 @@ inline bool operator==(const frag_info_t &l, const frag_info_t &r) {
   return memcmp(&l, &r, sizeof(l)) == 0;
 }
 
-ostream& operator<<(ostream &out, const frag_info_t &f);
+std::ostream& operator<<(std::ostream &out, const frag_info_t &f);
 
 
 struct nest_info_t : public scatter_info_t {
@@ -213,7 +220,7 @@ inline bool operator==(const nest_info_t &l, const nest_info_t &r) {
   return memcmp(&l, &r, sizeof(l)) == 0;
 }
 
-ostream& operator<<(ostream &out, const nest_info_t &n);
+std::ostream& operator<<(std::ostream &out, const nest_info_t &n);
 
 
 struct vinodeno_t {
@@ -258,7 +265,7 @@ CEPH_HASH_NAMESPACE_END
 
 
 
-inline ostream& operator<<(ostream &out, const vinodeno_t &vino) {
+inline std::ostream& operator<<(std::ostream &out, const vinodeno_t &vino) {
   out << vino.ino;
   if (vino.snapid == CEPH_NOSNAP)
     out << ".head";
@@ -295,7 +302,7 @@ inline void decode(client_writeable_range_t::byte_range_t& range, bufferlist::it
 
 WRITE_CLASS_ENCODER(client_writeable_range_t)
 
-ostream& operator<<(ostream& out, const client_writeable_range_t& r);
+std::ostream& operator<<(std::ostream& out, const client_writeable_range_t& r);
 
 inline bool operator==(const client_writeable_range_t& l,
 		       const client_writeable_range_t& r) {
@@ -338,7 +345,7 @@ struct inode_t {
   bufferlist inline_data;
   version_t  inline_version;
 
-  map<client_t,client_writeable_range_t> client_ranges;  // client(s) can write to these ranges
+  std::map<client_t,client_writeable_range_t> client_ranges;  // client(s) can write to these ranges
 
   // dirfrag, recursive accountin
   frag_info_t dirstat;         // protected by my filelock
@@ -403,7 +410,7 @@ struct inode_t {
 
   uint64_t get_max_size() const {
     uint64_t max = 0;
-      for (map<client_t,client_writeable_range_t>::const_iterator p = client_ranges.begin();
+      for (std::map<client_t,client_writeable_range_t>::const_iterator p = client_ranges.begin();
 	   p != client_ranges.end();
 	   ++p)
 	if (p->second.range.last > max)
@@ -414,7 +421,7 @@ struct inode_t {
     if (new_max == 0) {
       client_ranges.clear();
     } else {
-      for (map<client_t,client_writeable_range_t>::iterator p = client_ranges.begin();
+      for (std::map<client_t,client_writeable_range_t>::iterator p = client_ranges.begin();
 	   p != client_ranges.end();
 	   ++p)
 	p->second.range.last = new_max;
@@ -422,7 +429,7 @@ struct inode_t {
   }
 
   void trim_client_ranges(snapid_t last) {
-    map<client_t, client_writeable_range_t>::iterator p = client_ranges.begin();
+    std::map<client_t, client_writeable_range_t>::iterator p = client_ranges.begin();
     while (p != client_ranges.end()) {
       if (p->second.follows >= last)
 	client_ranges.erase(p++);
@@ -457,7 +464,7 @@ WRITE_CLASS_ENCODER(inode_t)
 struct old_inode_t {
   snapid_t first;
   inode_t inode;
-  map<string,bufferptr> xattrs;
+  std::map<string,bufferptr> xattrs;
 
   void encode(bufferlist &bl) const;
   void decode(bufferlist::iterator& bl);
@@ -496,7 +503,7 @@ struct old_rstat_t {
 };
 WRITE_CLASS_ENCODER(old_rstat_t)
 
-inline ostream& operator<<(ostream& out, const old_rstat_t& o) {
+inline std::ostream& operator<<(std::ostream& out, const old_rstat_t& o) {
   return out << "old_rstat(first " << o.first << " " << o.rstat << " " << o.accounted_rstat << ")";
 }
 
@@ -507,9 +514,10 @@ inline ostream& operator<<(ostream& out, const old_rstat_t& o) {
 
 struct session_info_t {
   entity_inst_t inst;
-  map<ceph_tid_t,inodeno_t> completed_requests;
+  std::map<ceph_tid_t,inodeno_t> completed_requests;
   interval_set<inodeno_t> prealloc_inos;   // preallocated, ready to use.
   interval_set<inodeno_t> used_inos;       // journaling use
+  std::map<std::string, std::string> client_metadata;
 
   client_t get_client() const { return client_t(inst.name.num()); }
 
@@ -576,7 +584,7 @@ struct dentry_key_t {
   }
 };
 
-inline ostream& operator<<(ostream& out, const dentry_key_t &k)
+inline std::ostream& operator<<(std::ostream& out, const dentry_key_t &k)
 {
   return out << "(" << k.name << "," << k.snapid << ")";
 }
@@ -614,7 +622,7 @@ inline bool operator<(const string_snap_t& l, const string_snap_t& r) {
   return c < 0 || (c == 0 && l.snapid < r.snapid);
 }
 
-inline ostream& operator<<(ostream& out, const string_snap_t &k)
+inline std::ostream& operator<<(std::ostream& out, const string_snap_t &k)
 {
   return out << "(" << k.name << "," << k.snapid << ")";
 }
@@ -657,7 +665,7 @@ struct metareqid_t {
 };
 WRITE_CLASS_ENCODER(metareqid_t)
 
-inline ostream& operator<<(ostream& out, const metareqid_t& r) {
+inline std::ostream& operator<<(std::ostream& out, const metareqid_t& r) {
   return out << r.name << ":" << r.tid;
 }
 
@@ -697,7 +705,8 @@ struct cap_reconnect_t {
   cap_reconnect_t() {
     memset(&capinfo, 0, sizeof(capinfo));
   }
-  cap_reconnect_t(uint64_t cap_id, inodeno_t pino, const string& p, int w, int i, inodeno_t sr) : 
+  cap_reconnect_t(uint64_t cap_id, inodeno_t pino, const string& p, int w, int i,
+		  inodeno_t sr, bufferlist& lb) :
     path(p) {
     capinfo.cap_id = cap_id;
     capinfo.wanted = w;
@@ -705,6 +714,7 @@ struct cap_reconnect_t {
     capinfo.snaprealm = sr;
     capinfo.pathbase = pino;
     capinfo.flock_len = 0;
+    flockbl.claim(lb);
   }
   void encode(bufferlist& bl) const;
   void decode(bufferlist::iterator& bl);
@@ -787,7 +797,7 @@ struct dirfrag_t {
 WRITE_CLASS_ENCODER(dirfrag_t)
 
 
-inline ostream& operator<<(ostream& out, const dirfrag_t df) {
+inline std::ostream& operator<<(std::ostream& out, const dirfrag_t df) {
   out << df.ino;
   if (!df.frag.is_root()) out << "." << df.frag;
   return out;
@@ -932,7 +942,7 @@ inline void decode(dirfrag_load_vec_t& c, const utime_t &t, bufferlist::iterator
   c.decode(t, p);
 }
 
-inline ostream& operator<<(ostream& out, dirfrag_load_vec_t& dl)
+inline std::ostream& operator<<(std::ostream& out, dirfrag_load_vec_t& dl)
 {
   // ugliness!
   utime_t now = ceph_clock_now(g_ceph_context);
@@ -984,7 +994,7 @@ inline void decode(mds_load_t &c, const utime_t &t, bufferlist::iterator &p) {
   c.decode(t, p);
 }
 
-inline ostream& operator<<( ostream& out, mds_load_t& load )
+inline std::ostream& operator<<( std::ostream& out, mds_load_t& load )
 {
   return out << "mdsload<" << load.auth << "/" << load.all
              << ", req " << load.req_rate 
@@ -1039,16 +1049,18 @@ class SimpleLock;
 
 class MDSCacheObject;
 
+typedef std::pair<mds_rank_t, mds_rank_t> mds_authority_t;
 // -- authority delegation --
 // directory authority types
 //  >= 0 is the auth mds
-#define CDIR_AUTH_PARENT   -1   // default
-#define CDIR_AUTH_UNKNOWN  -2
-#define CDIR_AUTH_DEFAULT   pair<int,int>(-1, -2)
-#define CDIR_AUTH_UNDEF     pair<int,int>(-2, -2)
+#define CDIR_AUTH_PARENT   mds_rank_t(-1)   // default
+#define CDIR_AUTH_UNKNOWN  mds_rank_t(-2)
+#define CDIR_AUTH_DEFAULT   mds_authority_t(CDIR_AUTH_PARENT, CDIR_AUTH_UNKNOWN)
+#define CDIR_AUTH_UNDEF     mds_authority_t(CDIR_AUTH_UNKNOWN, CDIR_AUTH_UNKNOWN)
 //#define CDIR_AUTH_ROOTINODE pair<int,int>( 0, -2)
 
 
+
 /*
  * for metadata leases to clients
  */
@@ -1073,10 +1085,10 @@ struct mdsco_db_line_prefix {
   MDSCacheObject *object;
   mdsco_db_line_prefix(MDSCacheObject *o) : object(o) {}
 };
-ostream& operator<<(ostream& out, mdsco_db_line_prefix o);
+std::ostream& operator<<(std::ostream& out, mdsco_db_line_prefix o);
 
 // printer
-ostream& operator<<(ostream& out, MDSCacheObject &o);
+std::ostream& operator<<(std::ostream& out, MDSCacheObject &o);
 
 class MDSCacheObjectInfo {
 public:
@@ -1156,8 +1168,8 @@ class MDSCacheObject {
   virtual ~MDSCacheObject() {}
 
   // printing
-  virtual void print(ostream& out) = 0;
-  virtual ostream& print_db_line_prefix(ostream& out) { 
+  virtual void print(std::ostream& out) = 0;
+  virtual std::ostream& print_db_line_prefix(std::ostream& out) { 
     return out << "mdscacheobject(" << this << ") "; 
   }
   
@@ -1180,7 +1192,7 @@ class MDSCacheObject {
 
   // --------------------------------------------
   // authority
-  virtual pair<int,int> authority() = 0;
+  virtual mds_authority_t authority() = 0;
   bool is_ambiguous_auth() {
     return authority().second != CDIR_AUTH_UNKNOWN;
   }
@@ -1190,7 +1202,7 @@ class MDSCacheObject {
 protected:
   __s32      ref;       // reference count
 #ifdef MDS_REF_SET
-  map<int,int> ref_map;
+  std::map<int,int> ref_map;
 #endif
 
  public:
@@ -1207,7 +1219,7 @@ protected:
 #ifdef MDS_REF_SET
   int get_pin_totals() {
     int total = 0;
-    for(map<int,int>::iterator i = ref_map.begin(); i != ref_map.end(); ++i) {
+    for(std::map<int,int>::iterator i = ref_map.begin(); i != ref_map.end(); ++i) {
       total += i->second;
     }
     return total;
@@ -1264,9 +1276,9 @@ protected:
 #endif
   }
 
-  void print_pin_set(ostream& out) {
+  void print_pin_set(std::ostream& out) {
 #ifdef MDS_REF_SET
-    map<int, int>::iterator it = ref_map.begin();
+    std::map<int, int>::iterator it = ref_map.begin();
     while (it != ref_map.end()) {
       out << " " << pin_name(it->first) << "=" << it->second;
       ++it;
@@ -1293,29 +1305,29 @@ protected:
   // replication (across mds cluster)
  protected:
   unsigned		replica_nonce; // [replica] defined on replica
-  map<int,unsigned>	replica_map;   // [auth] mds -> nonce
+  std::map<mds_rank_t,unsigned>	replica_map;   // [auth] mds -> nonce
 
  public:
   bool is_replicated() { return !replica_map.empty(); }
-  bool is_replica(int mds) { return replica_map.count(mds); }
+  bool is_replica(mds_rank_t mds) { return replica_map.count(mds); }
   int num_replicas() { return replica_map.size(); }
-  unsigned add_replica(int mds) {
+  unsigned add_replica(mds_rank_t mds) {
     if (replica_map.count(mds)) 
       return ++replica_map[mds];  // inc nonce
     if (replica_map.empty()) 
       get(PIN_REPLICATED);
     return replica_map[mds] = 1;
   }
-  void add_replica(int mds, unsigned nonce) {
+  void add_replica(mds_rank_t mds, unsigned nonce) {
     if (replica_map.empty()) 
       get(PIN_REPLICATED);
     replica_map[mds] = nonce;
   }
-  unsigned get_replica_nonce(int mds) {
+  unsigned get_replica_nonce(mds_rank_t mds) {
     assert(replica_map.count(mds));
     return replica_map[mds];
   }
-  void remove_replica(int mds) {
+  void remove_replica(mds_rank_t mds) {
     assert(replica_map.count(mds));
     replica_map.erase(mds);
     if (replica_map.empty())
@@ -1326,11 +1338,11 @@ protected:
       put(PIN_REPLICATED);
     replica_map.clear();
   }
-  map<int,unsigned>::iterator replicas_begin() { return replica_map.begin(); }
-  map<int,unsigned>::iterator replicas_end() { return replica_map.end(); }
-  const map<int,unsigned>& get_replicas() { return replica_map; }
-  void list_replicas(set<int>& ls) {
-    for (map<int,unsigned>::const_iterator p = replica_map.begin();
+  std::map<mds_rank_t,unsigned>::iterator replicas_begin() { return replica_map.begin(); }
+  std::map<mds_rank_t,unsigned>::iterator replicas_end() { return replica_map.end(); }
+  const std::map<mds_rank_t,unsigned>& get_replicas() { return replica_map; }
+  void list_replicas(std::set<mds_rank_t>& ls) {
+    for (std::map<mds_rank_t,unsigned>::const_iterator p = replica_map.begin();
 	 p != replica_map.end();
 	 ++p) 
       ls.insert(p->first);
@@ -1343,7 +1355,7 @@ protected:
   // ---------------------------------------------
   // waiting
  protected:
-  multimap<uint64_t, Context*>  waiting;
+  multimap<uint64_t, MDSInternalContextBase*>  waiting;
 
  public:
   bool is_waiter_for(uint64_t mask, uint64_t min=0) {
@@ -1352,7 +1364,7 @@ protected:
       while (min & (min-1))  // if more than one bit is set
 	min &= min-1;        //  clear LSB
     }
-    for (multimap<uint64_t,Context*>::iterator p = waiting.lower_bound(min);
+    for (multimap<uint64_t,MDSInternalContextBase*>::iterator p = waiting.lower_bound(min);
 	 p != waiting.end();
 	 ++p) {
       if (p->first & mask) return true;
@@ -1360,19 +1372,19 @@ protected:
     }
     return false;
   }
-  virtual void add_waiter(uint64_t mask, Context *c) {
+  virtual void add_waiter(uint64_t mask, MDSInternalContextBase *c) {
     if (waiting.empty())
       get(PIN_WAITER);
-    waiting.insert(pair<uint64_t,Context*>(mask, c));
+    waiting.insert(pair<uint64_t,MDSInternalContextBase*>(mask, c));
 //    pdout(10,g_conf->debug_mds) << (mdsco_db_line_prefix(this)) 
 //			       << "add_waiter " << hex << mask << dec << " " << c
 //			       << " on " << *this
 //			       << dendl;
     
   }
-  virtual void take_waiting(uint64_t mask, list<Context*>& ls) {
+  virtual void take_waiting(uint64_t mask, list<MDSInternalContextBase*>& ls) {
     if (waiting.empty()) return;
-    multimap<uint64_t,Context*>::iterator it = waiting.begin();
+    multimap<uint64_t,MDSInternalContextBase*>::iterator it = waiting.begin();
     while (it != waiting.end()) {
       if (it->first & mask) {
 	ls.push_back(it->second);
@@ -1394,7 +1406,7 @@ protected:
       put(PIN_WAITER);
   }
   void finish_waiting(uint64_t mask, int result = 0) {
-    list<Context*> finished;
+    list<MDSInternalContextBase*> finished;
     take_waiting(mask, finished);
     finish_contexts(g_ceph_context, finished, result);
   }
@@ -1408,7 +1420,7 @@ protected:
   virtual void encode_lock_state(int type, bufferlist& bl) { assert(0); }
   virtual void decode_lock_state(int type, bufferlist& bl) { assert(0); }
   virtual void finish_lock_waiters(int type, uint64_t mask, int r=0) { assert(0); }
-  virtual void add_lock_waiter(int type, uint64_t mask, Context *c) { assert(0); }
+  virtual void add_lock_waiter(int type, uint64_t mask, MDSInternalContextBase *c) { assert(0); }
   virtual bool is_lock_waiting(int type, uint64_t mask) { assert(0); return false; }
 
   virtual void clear_dirty_scattered(int type) { assert(0); }
@@ -1424,19 +1436,19 @@ protected:
 
 };
 
-inline ostream& operator<<(ostream& out, MDSCacheObject &o) {
+inline std::ostream& operator<<(std::ostream& out, MDSCacheObject &o) {
   o.print(out);
   return out;
 }
 
-inline ostream& operator<<(ostream& out, const MDSCacheObjectInfo &info) {
+inline std::ostream& operator<<(std::ostream& out, const MDSCacheObjectInfo &info) {
   if (info.ino) return out << info.ino << "." << info.snapid;
   if (info.dname.length()) return out << info.dirfrag << "/" << info.dname
 				      << " snap " << info.snapid;
   return out << info.dirfrag;
 }
 
-inline ostream& operator<<(ostream& out, mdsco_db_line_prefix o) {
+inline std::ostream& operator<<(std::ostream& out, mdsco_db_line_prefix o) {
   o.object->print_db_line_prefix(out);
   return out;
 }
