@@ -12,7 +12,7 @@
  * 
  */
 
-#include "msg/SimpleMessenger.h"
+#include "msg/Messenger.h"
 #include "messages/MMonGetMap.h"
 #include "messages/MMonGetVersion.h"
 #include "messages/MMonGetVersionReply.h"
@@ -114,11 +114,11 @@ int MonClient::get_monmap_privately()
   Mutex::Locker l(monc_lock);
 
   bool temp_msgr = false;
-  SimpleMessenger* smessenger = NULL;
+  Messenger* smessenger = NULL;
   if (!messenger) {
-    messenger = smessenger = new SimpleMessenger(cct,
-                                                 entity_name_t::CLIENT(-1),
-                                                 "temp_mon_client", getpid());
+    messenger = smessenger = Messenger::create(cct,
+					       entity_name_t::CLIENT(-1),
+					       "temp_mon_client", getpid());
     messenger->add_dispatcher_head(this);
     smessenger->start();
     temp_msgr = true;
@@ -153,6 +153,7 @@ int MonClient::get_monmap_privately()
     if (cur_con) {
       cur_con->mark_down();
       cur_con.reset(NULL);
+      cur_mon.clear();
     }
     monc_lock.Unlock();
     messenger->shutdown();
@@ -165,7 +166,6 @@ int MonClient::get_monmap_privately()
 
   hunting = true;  // reset this to true!
   cur_mon.clear();
-
   cur_con.reset(NULL);
 
   if (!monmap.fsid.is_zero())
@@ -218,9 +218,9 @@ int MonClient::ping_monitor(const string &mon_id, string *result_reply)
 
   MonClientPinger *pinger = new MonClientPinger(cct, result_reply);
 
-  Messenger *smsgr = new SimpleMessenger(cct,
-                                         entity_name_t::CLIENT(-1),
-                                         "temp_ping_client", getpid());
+  Messenger *smsgr = Messenger::create(cct,
+				       entity_name_t::CLIENT(-1),
+				       "temp_ping_client", getpid());
   smsgr->add_dispatcher_head(pinger);
   smsgr->start();
 
@@ -292,6 +292,7 @@ bool MonClient::ms_dispatch(Message *m)
   case MSG_LOGACK:
     if (log_client) {
       log_client->handle_log_ack(static_cast<MLogAck*>(m));
+      m->put();
       if (more_log_pending) {
 	send_log();
       }
@@ -397,7 +398,7 @@ int MonClient::init()
 
 void MonClient::shutdown()
 {
-  ldout(cct, 10) << __func__ << "shutdown" << dendl;
+  ldout(cct, 10) << __func__ << dendl;
   monc_lock.Lock();
   while (!version_requests.empty()) {
     version_requests.begin()->second->context->complete(-ECANCELED);
@@ -424,6 +425,7 @@ void MonClient::shutdown()
   if (cur_con)
     cur_con->mark_down();
   cur_con.reset(NULL);
+  cur_mon.clear();
 
   monc_lock.Unlock();
 }
