@@ -8,7 +8,7 @@
 #include <string>
 
 #include <vector>
-#include <tr1/memory>
+#include "include/memory.h"
 #include <boost/scoped_ptr.hpp>
 
 #include "ObjectMap.h"
@@ -16,6 +16,8 @@
 #include "osd/osd_types.h"
 #include "common/Mutex.h"
 #include "common/Cond.h"
+#include "common/simple_cache.hpp"
+#include <boost/optional.hpp>
 
 /**
  * DBObjectMap: Implements ObjectMap in terms of KeyValueDB
@@ -68,8 +70,52 @@ public:
   set<uint64_t> in_use;
   set<ghobject_t> map_header_in_use;
 
-  DBObjectMap(KeyValueDB *db) : db(db),
-				header_lock("DBOBjectMap")
+  /**
+   * Takes the map_header_in_use entry in constructor, releases in
+   * destructor
+   */
+  class MapHeaderLock {
+    DBObjectMap *db;
+    boost::optional<ghobject_t> locked;
+
+    MapHeaderLock(const MapHeaderLock &);
+    MapHeaderLock &operator=(const MapHeaderLock &);
+  public:
+    MapHeaderLock(DBObjectMap *db) : db(db) {}
+    MapHeaderLock(DBObjectMap *db, const ghobject_t &oid) : db(db), locked(oid) {
+      Mutex::Locker l(db->header_lock);
+      while (db->map_header_in_use.count(*locked))
+	db->map_header_cond.Wait(db->header_lock);
+      db->map_header_in_use.insert(*locked);
+    }
+
+    const ghobject_t &get_locked() const {
+      assert(locked);
+      return *locked;
+    }
+
+    void swap(MapHeaderLock &o) {
+      assert(db == o.db);
+
+      // centos6's boost optional doesn't seem to have swap :(
+      boost::optional<ghobject_t> _locked = o.locked;
+      o.locked = locked;
+      locked = _locked;
+    }
+
+    ~MapHeaderLock() {
+      if (locked) {
+	Mutex::Locker l(db->header_lock);
+	assert(db->map_header_in_use.count(*locked));
+	db->map_header_cond.Signal();
+	db->map_header_in_use.erase(*locked);
+      }
+    }
+  };
+
+  DBObjectMap(KeyValueDB *db) : db(db), header_lock("DBOBjectMap"),
+                                cache_lock("DBObjectMap::CacheLock"),
+                                caches(g_conf->filestore_omap_header_cache_size)
     {}
 
   int set_keys(
@@ -280,7 +326,9 @@ public:
 				   coll_t *c, ghobject_t *oid);
 private:
   /// Implicit lock on Header->seq
-  typedef std::tr1::shared_ptr<_Header> Header;
+  typedef ceph::shared_ptr<_Header> Header;
+  Mutex cache_lock;
+  SimpleLRU<ghobject_t, _Header> caches;
 
   string map_header_key(const ghobject_t &oid);
   string header_key(uint64_t seq);
@@ -312,16 +360,18 @@ private:
   public:
     DBObjectMap *map;
 
+    /// NOTE: implicit lock hlock->get_locked() when returned out of the class
+    MapHeaderLock hlock;
     /// NOTE: implicit lock on header->seq AND for all ancestors
     Header header;
 
     /// parent_iter == NULL iff no parent
-    std::tr1::shared_ptr<DBObjectMapIteratorImpl> parent_iter;
+    ceph::shared_ptr<DBObjectMapIteratorImpl> parent_iter;
     KeyValueDB::Iterator key_iter;
     KeyValueDB::Iterator complete_iter;
 
     /// cur_iter points to currently valid iterator
-    std::tr1::shared_ptr<ObjectMapIteratorImpl> cur_iter;
+    ceph::shared_ptr<ObjectMapIteratorImpl> cur_iter;
     int r;
 
     /// init() called, key_iter, complete_iter, parent_iter filled in
@@ -330,7 +380,7 @@ private:
     bool invalid;
 
     DBObjectMapIteratorImpl(DBObjectMap *map, Header header) :
-      map(map), header(header), r(0), ready(false), invalid(true) {}
+      map(map), hlock(map), header(header), r(0), ready(false), invalid(true) {}
     int seek_to_first();
     int seek_to_last();
     int upper_bound(const string &after);
@@ -360,7 +410,7 @@ private:
     int adjust();
   };
 
-  typedef std::tr1::shared_ptr<DBObjectMapIteratorImpl> DBObjectMapIterator;
+  typedef ceph::shared_ptr<DBObjectMapIteratorImpl> DBObjectMapIterator;
   DBObjectMapIterator _get_iterator(Header header) {
     return DBObjectMapIterator(new DBObjectMapIteratorImpl(this, header));
   }
@@ -374,13 +424,17 @@ private:
   void set_header(Header input, KeyValueDB::Transaction t);
 
   /// Remove leaf node corresponding to oid in c
-  void remove_map_header(const ghobject_t &oid,
-			 Header header,
-			 KeyValueDB::Transaction t);
+  void remove_map_header(
+    const MapHeaderLock &l,
+    const ghobject_t &oid,
+    Header header,
+    KeyValueDB::Transaction t);
 
   /// Set leaf node for c and oid to the value of header
-  void set_map_header(const ghobject_t &oid, _Header header,
-		      KeyValueDB::Transaction t);
+  void set_map_header(
+    const MapHeaderLock &l,
+    const ghobject_t &oid, _Header header,
+    KeyValueDB::Transaction t);
 
   /// Set leaf node for c and oid to the value of header
   bool check_spos(const ghobject_t &oid,
@@ -388,8 +442,10 @@ private:
 		  const SequencerPosition *spos);
 
   /// Lookup or create header for c oid
-  Header lookup_create_map_header(const ghobject_t &oid,
-				  KeyValueDB::Transaction t);
+  Header lookup_create_map_header(
+    const MapHeaderLock &l,
+    const ghobject_t &oid,
+    KeyValueDB::Transaction t);
 
   /**
    * Generate new header for c oid with new seq number
@@ -403,10 +459,14 @@ private:
   }
 
   /// Lookup leaf header for c oid
-  Header _lookup_map_header(const ghobject_t &oid);
-  Header lookup_map_header(const ghobject_t &oid) {
+  Header _lookup_map_header(
+    const MapHeaderLock &l,
+    const ghobject_t &oid);
+  Header lookup_map_header(
+    const MapHeaderLock &l2,
+    const ghobject_t &oid) {
     Mutex::Locker l(header_lock);
-    return _lookup_map_header(oid);
+    return _lookup_map_header(l2, oid);
   }
 
   /// Lookup header node for input
@@ -447,25 +507,8 @@ private:
 		   KeyValueDB::Transaction t);
 
   /** 
-   * Removes map header lock once Header is out of scope
-   * @see lookup_map_header
-   */
-  class RemoveMapHeaderOnDelete {
-  public:
-    DBObjectMap *db;
-    ghobject_t oid;
-    RemoveMapHeaderOnDelete(DBObjectMap *db, const ghobject_t &oid) :
-      db(db), oid(oid) {}
-    void operator() (_Header *header) {
-      Mutex::Locker l(db->header_lock);
-      db->map_header_in_use.erase(oid);
-      db->map_header_cond.Signal();
-      delete header;
-    }
-  };
-
-  /** 
-   * Removes header seq lock once Header is out of scope
+   * Removes header seq lock and possibly object lock
+   * once Header is out of scope
    * @see lookup_parent
    * @see generate_new_header
    */
@@ -476,6 +519,7 @@ private:
       db(db) {}
     void operator() (_Header *header) {
       Mutex::Locker l(db->header_lock);
+      assert(db->in_use.count(header->seq));
       db->in_use.erase(header->seq);
       db->header_cond.Signal();
       delete header;
