@@ -41,14 +41,6 @@ using namespace std;
 
 #include "include/uuid.h"
 
-enum kvstore_types {
-    KV_TYPE_NONE = 0,
-    KV_TYPE_LEVELDB,
-    KV_TYPE_KINETIC,
-    KV_TYPE_OTHER
-};
-
-
 static uint64_t default_strip_size = 1024;
 
 class StripObjectMap: public GenericObjectMap {
@@ -74,10 +66,11 @@ class StripObjectMap: public GenericObjectMap {
                    // also block read operation which not should be permitted.
     coll_t cid;
     ghobject_t oid;
+    bool updated;
     bool deleted;
     map<pair<string, string>, bufferlist> buffers;  // pair(prefix, key)
 
-    StripObjectHeader(): strip_size(default_strip_size), max_size(0), deleted(false) {}
+    StripObjectHeader(): strip_size(default_strip_size), max_size(0), updated(false), deleted(false) {}
 
     void encode(bufferlist &bl) const {
       ENCODE_START(1, 1, bl);
@@ -145,6 +138,27 @@ class StripObjectMap: public GenericObjectMap {
 };
 
 
+class KVSuperblock {
+public:
+  CompatSet compat_features;
+  string backend;
+
+  KVSuperblock() { }
+
+  void encode(bufferlist &bl) const;
+  void decode(bufferlist::iterator &bl);
+  void dump(Formatter *f) const;
+  static void generate_test_instances(list<KVSuperblock*>& o);
+};
+WRITE_CLASS_ENCODER(KVSuperblock)
+
+
+inline ostream& operator<<(ostream& out, const KVSuperblock& sb)
+{
+  return out << "sb(" << sb.compat_features << " " << sb.backend << ")";
+}
+
+
 class KeyValueStore : public ObjectStore,
                       public md_config_obs_t {
  public:
@@ -186,8 +200,6 @@ class KeyValueStore : public ObjectStore,
 
   int fsid_fd, current_fd;
 
-  enum kvstore_types kv_type;
-
   deque<uint64_t> snaps;
 
   // ObjectMap
@@ -466,23 +478,13 @@ class KeyValueStore : public ObjectStore,
                 bool update_to=false);
   ~KeyValueStore();
 
-  int _detect_backend() {
-    if (g_conf->osd_keyvaluedb == "leveldb")
-      kv_type = KV_TYPE_LEVELDB;
-#ifdef HAVE_KINETIC
-    else if (g_conf->osd_keyvaluedb == "kinetic")
-      kv_type = KV_TYPE_KINETIC;
-#endif
-    else
-      return -EINVAL;
-    return 0;
-  }
   bool test_mount_in_use();
   int version_stamp_is_valid(uint32_t *version);
   int update_version_stamp();
   uint32_t get_target_version() {
     return target_version;
   }
+  bool need_journal() { return false; };
   int peek_journal_fsid(uuid_d *id) {
     *id = fsid;
     return 0;
@@ -597,6 +599,8 @@ class KeyValueStore : public ObjectStore,
                            BufferTransaction &t);
 
   // collections
+  int _collection_hint_expected_num_objs(coll_t cid, uint32_t pg_num,
+      uint64_t num_objs) const { return 0; }
   int _create_collection(coll_t c, BufferTransaction &t);
   int _destroy_collection(coll_t c, BufferTransaction &t);
   int _collection_add(coll_t c, coll_t ocid, const ghobject_t& oid,
@@ -680,6 +684,25 @@ class KeyValueStore : public ObjectStore,
   static const string COLLECTION;
   static const string COLLECTION_ATTR;
   static const uint32_t COLLECTION_VERSION = 1;
+
+  KVSuperblock superblock;
+  /**
+   * write_superblock()
+   *
+   * Write superblock to persisent storage
+   *
+   * return value: 0 on success, otherwise negative errno
+   */
+  int write_superblock();
+
+  /**
+   * read_superblock()
+   *
+   * Fill in KeyValueStore::superblock by reading persistent storage
+   *
+   * return value: 0 on success, otherwise negative errno
+   */
+  int read_superblock();
 };
 
 WRITE_CLASS_ENCODER(StripObjectMap::StripObjectHeader)
