@@ -22,7 +22,6 @@
 #include "PGLog.h"
 #include "osd_types.h"
 #include "common/WorkQueue.h"
-#include "osd_types.h"
 #include "include/Context.h"
 #include "os/ObjectStore.h"
 #include "common/LogClient.h"
@@ -191,7 +190,7 @@
      virtual void update_stats(
        const pg_stat_t &stat) = 0;
 
-     virtual void schedule_work(
+     virtual void schedule_recovery_work(
        GenContext<ThreadPool::TPHandle&> *c) = 0;
 
      virtual pg_shard_t whoami_shard() const = 0;
@@ -599,7 +598,7 @@
      map<hobject_t, pg_shard_t> &authoritative,
      map<hobject_t, set<pg_shard_t> > &invalid_snapcolls,
      int &shallow_errors, int &deep_errors,
-     const spg_t pgid,
+     const spg_t& pgid,
      const vector<int> &acting,
      ostream &errorstream);
    virtual uint64_t be_get_ondisk_size(
@@ -632,14 +631,14 @@ struct PG_SendMessageOnConn: public Context {
   }
 };
 
-struct PG_QueueAsync : public Context {
+struct PG_RecoveryQueueAsync : public Context {
   PGBackend::Listener *pg;
   GenContext<ThreadPool::TPHandle&> *c;
-  PG_QueueAsync(
+  PG_RecoveryQueueAsync(
     PGBackend::Listener *pg,
     GenContext<ThreadPool::TPHandle&> *c) : pg(pg), c(c) {}
   void finish(int) {
-    pg->schedule_work(c);
+    pg->schedule_recovery_work(c);
   }
 };
 
