@@ -158,6 +158,7 @@ bool ReplicatedBackend::handle_message(
       }
     } else {
       sub_op_modify(op);
+      return true;
     }
     break;
   }
@@ -174,6 +175,7 @@ bool ReplicatedBackend::handle_message(
       }
     } else {
       sub_op_modify_reply(op);
+      return true;
     }
     break;
   }
@@ -192,8 +194,9 @@ void ReplicatedBackend::clear_state()
   pull_from_peer.clear();
 }
 
-void ReplicatedBackend::_on_change(ObjectStore::Transaction *t)
+void ReplicatedBackend::on_change()
 {
+  dout(10) << __func__ << dendl;
   for (map<ceph_tid_t, InProgressOp>::iterator i = in_progress_ops.begin();
        i != in_progress_ops.end();
        in_progress_ops.erase(i++)) {
@@ -254,14 +257,14 @@ void ReplicatedBackend::objects_read_async(
     int _r = store->read(coll, hoid, i->first.first,
 			 i->first.second, *(i->second.first));
     if (i->second.second) {
-      get_parent()->schedule_work(
+      get_parent()->schedule_recovery_work(
 	get_parent()->bless_gencontext(
 	  new AsyncReadCallback(_r, i->second.second)));
     }
     if (_r < 0)
       r = _r;
   }
-  get_parent()->schedule_work(
+  get_parent()->schedule_recovery_work(
     get_parent()->bless_gencontext(
       new AsyncReadCallback(r, on_complete)));
 }
@@ -273,6 +276,7 @@ class RPGTransaction : public PGBackend::PGTransaction {
   set<hobject_t> temp_added;
   set<hobject_t> temp_cleared;
   ObjectStore::Transaction *t;
+  uint64_t written;
   const coll_t &get_coll_ct(const hobject_t &hoid) {
     if (hoid.is_temp()) {
       temp_cleared.erase(hoid);
@@ -295,7 +299,7 @@ class RPGTransaction : public PGBackend::PGTransaction {
   }
 public:
   RPGTransaction(coll_t coll, coll_t temp_coll)
-    : coll(coll), temp_coll(temp_coll), t(new ObjectStore::Transaction)
+    : coll(coll), temp_coll(temp_coll), t(new ObjectStore::Transaction), written(0)
     {}
 
   /// Yields ownership of contained transaction
@@ -317,6 +321,7 @@ public:
     uint64_t len,
     bufferlist &bl
     ) {
+    written += len;
     t->write(get_coll_ct(hoid), hoid, off, len, bl);
   }
   void remove(
@@ -329,7 +334,7 @@ public:
     version_t former_version) {
     t->collection_move_rename(
       coll, hoid, coll,
-      ghobject_t(hoid, former_version, ghobject_t::NO_SHARD));
+      ghobject_t(hoid, former_version, shard_id_t::NO_SHARD));
   }
   void setattrs(
     const hobject_t &hoid,
@@ -354,6 +359,8 @@ public:
     const hobject_t &hoid,
     map<string, bufferlist> &keys
     ) {
+    for (map<string, bufferlist>::iterator p = keys.begin(); p != keys.end(); ++p)
+      written += p->first.length() + p->second.length();
     return t->omap_setkeys(get_coll(hoid), hoid, keys);
   }
   void omap_rmkeys(
@@ -371,6 +378,7 @@ public:
     const hobject_t &hoid,
     bufferlist &header
     ) {
+    written += header.length();
     t->omap_setheader(get_coll(hoid), hoid, header);
   }
   void clone_range(
@@ -435,6 +443,8 @@ public:
     ) {
     RPGTransaction *to_append = dynamic_cast<RPGTransaction*>(_to_append);
     assert(to_append);
+    written += to_append->written;
+    to_append->written = 0;
     t->append(*(to_append->t));
     for (set<hobject_t>::iterator i = to_append->temp_added.begin();
 	 i != to_append->temp_added.end();
@@ -456,7 +466,7 @@ public:
     return t->empty();
   }
   uint64_t get_bytes_written() const {
-    return t->get_encoded_bytes();
+    return written;
   }
   ~RPGTransaction() { delete t; }
 };
@@ -493,7 +503,9 @@ void ReplicatedBackend::submit_transaction(
   const eversion_t &at_version,
   PGTransaction *_t,
   const eversion_t &trim_to,
+  const eversion_t &trim_rollback_to,
   vector<pg_log_entry_t> &log_entries,
+  boost::optional<pg_hit_set_history_t> &hset_history,
   Context *on_local_applied_sync,
   Context *on_all_acked,
   Context *on_all_commit,
@@ -532,10 +544,12 @@ void ReplicatedBackend::submit_transaction(
     tid,
     reqid,
     trim_to,
+    trim_rollback_to,
     t->get_temp_added().size() ? *(t->get_temp_added().begin()) : hobject_t(),
     t->get_temp_cleared().size() ?
       *(t->get_temp_cleared().begin()) :hobject_t(),
     log_entries,
+    hset_history,
     &op,
     op_t);
 
@@ -546,7 +560,13 @@ void ReplicatedBackend::submit_transaction(
   }
   clear_temp_objs(t->get_temp_cleared());
 
-  parent->log_operation(log_entries, trim_to, true, &local_t);
+  parent->log_operation(
+    log_entries,
+    hset_history,
+    trim_to,
+    trim_rollback_to,
+    true,
+    &local_t);
   local_t.append(*op_t);
   local_t.swap(*op_t);
   
