@@ -1627,7 +1627,7 @@ public:
     messenger(m), monc(mc),
     osdmap(new OSDMap),
     cct(cct_),
-    initialized(false),
+    initialized(0),
     last_tid(0), client_inc(-1), max_linger_id(0),
     num_unacked(0), num_uncommitted(0),
     global_op_flags(0),
@@ -1687,12 +1687,14 @@ public:
  public:
   bool ms_dispatch(Message *m);
   bool ms_can_fast_dispatch_any() const {
-    return true;
+    return false;
   }
   bool ms_can_fast_dispatch(Message *m) const {
     switch (m->get_type()) {
     case CEPH_MSG_OSD_OPREPLY:
-      return true;
+      /* sadly, we need to solve a deadlock before reenabling.
+       * See tracker issue #9462 */
+      return false;
     default:
       return false;
     }
