@@ -151,7 +151,7 @@ TEST(Buffer, constructors) {
 
     unsigned zc_len = 4;
     ::unlink("testfile");
-    ::system("echo ABC > testfile");
+    EXPECT_EQ(0, ::system("echo ABC > testfile"));
     int fd = ::open("testfile", O_RDONLY);
     bufferptr ptr(buffer::create_zero_copy(zc_len, fd, NULL));
     EXPECT_EQ(zc_len, ptr.length());
@@ -179,7 +179,7 @@ protected:
   virtual void SetUp() {
     len = 4;
     ::unlink("testfile");
-    ::system("echo ABC > testfile");
+    EXPECT_EQ(0, ::system("echo ABC > testfile"));
     fd = ::open("testfile", O_RDONLY);
     assert(fd >= 0);
   }
@@ -1122,6 +1122,52 @@ TEST(BufferList, contents_equal) {
   ASSERT_FALSE(bl1.contents_equal(bl3)); // same length different content
 }
 
+TEST(BufferList, is_aligned) {
+  const int SIMD_ALIGN = 32;
+  {
+    bufferlist bl;
+    EXPECT_TRUE(bl.is_aligned(SIMD_ALIGN));
+  }
+  {
+    bufferlist bl;
+    bufferptr ptr(buffer::create_aligned(2, SIMD_ALIGN));
+    ptr.set_offset(1);
+    ptr.set_length(1);
+    bl.append(ptr);
+    EXPECT_FALSE(bl.is_aligned(SIMD_ALIGN));
+    bl.rebuild_aligned(SIMD_ALIGN);
+    EXPECT_TRUE(bl.is_aligned(SIMD_ALIGN));
+  }
+  {
+    bufferlist bl;
+    bufferptr ptr(buffer::create_aligned(SIMD_ALIGN + 1, SIMD_ALIGN));
+    ptr.set_offset(1);
+    ptr.set_length(SIMD_ALIGN);
+    bl.append(ptr);
+    EXPECT_FALSE(bl.is_aligned(SIMD_ALIGN));
+    bl.rebuild_aligned(SIMD_ALIGN);
+    EXPECT_TRUE(bl.is_aligned(SIMD_ALIGN));
+  }
+}
+
+TEST(BufferList, is_n_align_sized) {
+  const int SIMD_ALIGN = 32;
+  {
+    bufferlist bl;
+    EXPECT_TRUE(bl.is_n_align_sized(SIMD_ALIGN));
+  }
+  {
+    bufferlist bl;
+    bl.append_zero(1);
+    EXPECT_FALSE(bl.is_n_align_sized(SIMD_ALIGN));
+  }
+  {
+    bufferlist bl;
+    bl.append_zero(SIMD_ALIGN);
+    EXPECT_TRUE(bl.is_n_align_sized(SIMD_ALIGN));
+  }
+}
+
 TEST(BufferList, is_page_aligned) {
   {
     bufferlist bl;
@@ -1322,6 +1368,14 @@ TEST(BufferList, rebuild_page_aligned) {
     EXPECT_TRUE(bl.is_page_aligned());
     EXPECT_EQ((unsigned)1, bl.buffers().size());
   }
+  {
+    bufferlist bl;
+    bufferptr ptr(buffer::create_page_aligned(1));
+    char *p = ptr.c_str();
+    bl.append(ptr);
+    bl.rebuild_page_aligned();
+    EXPECT_EQ(p, bl.buffers().front().c_str());
+  }
   {
     bufferlist bl;
     {
@@ -1757,9 +1811,9 @@ TEST(BufferList, read_file) {
   bufferlist bl;
   ::unlink("testfile");
   EXPECT_EQ(-ENOENT, bl.read_file("UNLIKELY", &error));
-  ::system("echo ABC > testfile ; chmod 0 testfile");
+  EXPECT_EQ(0, ::system("echo ABC > testfile ; chmod 0 testfile"));
   EXPECT_EQ(-EACCES, bl.read_file("testfile", &error));
-  ::system("chmod +r testfile");
+  EXPECT_EQ(0, ::system("chmod +r testfile"));
   EXPECT_EQ(0, bl.read_file("testfile", &error));
   ::unlink("testfile");
   EXPECT_EQ((unsigned)4, bl.length());
@@ -1770,7 +1824,7 @@ TEST(BufferList, read_file) {
 TEST(BufferList, read_fd) {
   unsigned len = 4;
   ::unlink("testfile");
-  ::system("echo ABC > testfile");
+  EXPECT_EQ(0, ::system("echo ABC > testfile"));
   int fd = -1;
   bufferlist bl;
   EXPECT_EQ(-EBADF, bl.read_fd(fd, len));
