@@ -14,11 +14,11 @@ set -e
 [ -z "$CEPH_NUM_RGW" ] && CEPH_NUM_RGW="$RGW"
 
 [ -z "$CEPH_NUM_MON" ] && CEPH_NUM_MON=3
-[ -z "$CEPH_NUM_OSD" ] && CEPH_NUM_OSD=1
+[ -z "$CEPH_NUM_OSD" ] && CEPH_NUM_OSD=3
 [ -z "$CEPH_NUM_MDS" ] && CEPH_NUM_MDS=3
 [ -z "$CEPH_NUM_RGW" ] && CEPH_NUM_RGW=1
 
-[ -z "$CEPH_DIR" ] && CEPH_DIR="$PWD/"
+[ -z "$CEPH_DIR" ] && CEPH_DIR="$PWD"
 [ -z "$CEPH_DEV_DIR" ] && CEPH_DEV_DIR="$CEPH_DIR/dev"
 [ -z "$CEPH_OUT_DIR" ] && CEPH_OUT_DIR="$CEPH_DIR/out"
 [ -z "$CEPH_RGW_PORT" ] && CEPH_RGW_PORT=8000
@@ -35,16 +35,17 @@ start_rgw=0
 ip=""
 nodaemon=0
 smallmds=0
+ec=0
 hitset=""
 overwrite_conf=1
 cephx=1 #turn cephx on by default
 cache=""
 memstore=0
+journal=1
 
 MON_ADDR=""
 
-conf="$CEPH_DIR/ceph.conf"
-
+conf_fn="$CEPH_DIR/ceph.conf"
 keyring_fn="$CEPH_DIR/keyring"
 osdmap_fn="/tmp/ceph_osdmap.$$"
 monmap_fn="/tmp/ceph_monmap.$$"
@@ -65,7 +66,10 @@ usage=$usage"\t-k keep old configuration files\n"
 usage=$usage"\t-x enable cephx (on by default)\n"
 usage=$usage"\t-X disable cephx\n"
 usage=$usage"\t--hitset <pool> <hit_set_type>: enable hitset tracking\n"
+usage=$usage"\t-e : create an erasure pool\n";
 usage=$usage"\t-o config\t\t add extra config parameters to mds section\n"
+usage=$usage"\t-J no journal\t\tdisable filestore journal\n"
+
 
 usage_exit() {
 	printf "$usage"
@@ -91,6 +95,9 @@ case $1 in
     -r )
 	    start_rgw=1
 	    ;;
+    -e )
+	    ec=1
+	    ;;
     --new | -n )
 	    new=1
 	    ;;
@@ -143,6 +150,9 @@ case $1 in
     -X )
 	    cephx=0
 	    ;;
+    -J )
+	    journal=0
+	    ;;
     -k )
 	    overwrite_conf=0
 	    ;;
@@ -179,7 +189,7 @@ if [ "$start_all" -eq 1 ]; then
 	start_osd=1
 fi
 
-ARGS="-c $conf"
+ARGS="-c $conf_fn"
 
 run() {
     type=$1
@@ -224,6 +234,7 @@ else
         debug monc = 20
         debug journal = 20
         debug filestore = 20
+        debug rgw = 20
         debug objclass = 20'
     CMDSDEBUG='
         debug ms = 1
@@ -287,9 +298,9 @@ echo "ip $IP"
 
 
 if [ "$cephx" -eq 1 ]; then
-    CEPH_ADM="$CEPH_BIN/ceph -c $conf -k $keyring_fn"
+    CEPH_ADM="$CEPH_BIN/ceph -c $conf_fn -k $keyring_fn"
 else
-    CEPH_ADM="$CEPH_BIN/ceph -c $conf"
+    CEPH_ADM="$CEPH_BIN/ceph -c $conf_fn"
 fi
 
 MONS=""
@@ -319,7 +330,7 @@ if [ "$start_mon" -eq 1 ]; then
 
 	if [ "$new" -eq 1 ]; then
 		if [ $overwrite_conf -eq 1 ]; then
-		        cat <<EOF > $conf
+		        cat <<EOF > $conf_fn
 ; generated by vstart.sh on `date`
 [global]
         fsid = $(uuidgen)
@@ -329,24 +340,32 @@ if [ "$start_mon" -eq 1 ]; then
         osd pool default min size = 1
         osd pool default erasure code directory = .libs
         osd pool default erasure code profile = plugin=jerasure technique=reed_sol_van k=2 m=1 ruleset-failure-domain=osd
+        rgw frontends = fastcgi, civetweb port=$CEPH_RGW_PORT
+        filestore fd cache size = 32
         run dir = $CEPH_OUT_DIR
 EOF
 if [ "$cephx" -eq 1 ] ; then
-cat <<EOF >> $conf
+cat <<EOF >> $conf_fn
         auth supported = cephx
 EOF
 else
-cat <<EOF >> $conf
+cat <<EOF >> $conf_fn
 	auth cluster required = none
 	auth service required = none
 	auth client required = none
 EOF
 fi
-			cat <<EOF >> $conf
+                        if [ $journal -eq 1 ]; then
+			    journal_path="$CEPH_DEV_DIR/osd\$id.journal"
+			else
+			    journal_path=""
+			fi
+			cat <<EOF >> $conf_fn
 
 [client]
         keyring = $keyring_fn
         log file = $CEPH_OUT_DIR/\$name.\$pid.log
+        admin socket = $CEPH_OUT_DIR/\$name.\$pid.asok
 
 [mds]
 $DAEMONOPTS
@@ -359,18 +378,25 @@ $extra_conf
 [osd]
 $DAEMONOPTS
         osd data = $CEPH_DEV_DIR/osd\$id
-        osd journal = $CEPH_DEV_DIR/osd\$id.journal
+        osd journal = $journal_path
         osd journal size = 100
         osd class tmp = out
         osd class dir = .libs
         osd scrub load threshold = 5.0
         osd debug op order = true
+        filestore wbthrottle xfs ios start flusher = 10
+        filestore wbthrottle xfs ios hard limit = 20
+        filestore wbthrottle xfs inodes hard limit = 30
+        filestore wbthrottle btrfs ios start flusher = 10
+        filestore wbthrottle btrfs ios hard limit = 20
+        filestore wbthrottle btrfs inodes hard limit = 30
 $COSDDEBUG
 $COSDMEMSTORE
 $extra_conf
 [mon]
         mon pg warn min per osd = 10
         mon osd allow primary affinity = true
+        mon reweight min pgs per osd = 4
 $DAEMONOPTS
 $CMONDEBUG
 $extra_conf
@@ -393,7 +419,7 @@ EOF
 	        $SUDO $CEPH_BIN/ceph-authtool --gen-key --name=client.admin --set-uid=0 \
 		    --cap mon 'allow *' \
 		    --cap osd 'allow *' \
-		    --cap mds allow \
+		    --cap mds 'allow *' \
 		    $keyring_fn
 
 		# build a fresh fs monmap, mon fs
@@ -403,7 +429,7 @@ EOF
 		do
 			str=$str" --add $f $IP:$(($CEPH_PORT+$count))"
 			if [ $overwrite_conf -eq 1 ]; then
-				cat <<EOF >> $conf
+				cat <<EOF >> $conf_fn
 [mon.$f]
         host = $HOSTNAME
         mon data = $CEPH_DEV_DIR/mon.$f
@@ -424,7 +450,7 @@ EOF
                     cmd="mkdir -p $CEPH_DEV_DIR/mon.$f"
                     echo $cmd
                     $cmd
-		    cmd="$CEPH_BIN/ceph-mon --mkfs -c $conf -i $f --monmap=$monmap_fn"
+		    cmd="$CEPH_BIN/ceph-mon --mkfs -c $conf_fn -i $f --monmap=$monmap_fn"
 		    cmd="$cmd --keyring=$keyring_fn"
 		    echo $cmd
 		    $cmd
@@ -448,7 +474,7 @@ if [ "$start_osd" -eq 1 ]; then
     do
 	if [ "$new" -eq 1 ]; then
 	    if [ $overwrite_conf -eq 1 ]; then
-		    cat <<EOF >> $conf
+		    cat <<EOF >> $conf_fn
 [osd.$osd]
         host = $HOSTNAME
 EOF
@@ -474,7 +500,7 @@ fi
 
 # mds
 if [ "$smallmds" -eq 1 ]; then
-    cat <<EOF >> $conf
+    cat <<EOF >> $conf_fn
 [mds]
 	mds log max segments = 2
 	mds cache size = 10000
@@ -489,13 +515,13 @@ if [ "$start_mds" -eq 1 -a "$CEPH_NUM_MDS" -gt 0 ]; then
 	    mkdir -p $CEPH_DEV_DIR/mds.$name
 	    key_fn=$CEPH_DEV_DIR/mds.$name/keyring
 	    if [ $overwrite_conf -eq 1 ]; then
-	    	cat <<EOF >> $conf
+	    cat <<EOF >> $conf_fn
 [mds.$name]
         host = $HOSTNAME
 EOF
 		if [ "$standby" -eq 1 ]; then
 		    mkdir -p $CEPH_DEV_DIR/mds.${name}s
-		    cat <<EOF >> $conf
+		    cat <<EOF >> $conf_fn
        mds standby for rank = $mds
 [mds.${name}s]
         mds standby replay = true
@@ -511,6 +537,18 @@ EOF
                     $SUDO $CEPH_ADM -i $CEPH_DEV_DIR/mds.${name}s/keyring auth add mds.${name}s \
 			mon 'allow *' osd 'allow *' mds 'allow'
 	    fi
+
+        cmd="$CEPH_ADM osd pool create cephfs_data 8"
+        echo $cmd
+        $cmd
+
+        cmd="$CEPH_ADM osd pool create cephfs_metadata 8"
+        echo $cmd
+        $cmd
+
+        cmd="$CEPH_ADM fs new cephfs cephfs_metadata cephfs_data"
+        echo $cmd
+        $cmd
 	fi
 	
 	run 'mds' $CEPH_BIN/ceph-mds -i $name $ARGS $CMDS_ARGS
@@ -530,90 +568,14 @@ EOF
     $cmd
 fi
 
-
-# rgw
-if [ "$start_rgw" -eq 1 ]; then
-    for rgw in `seq 0 $((CEPH_NUM_RGW-1))`
-    do
-	rgwport=$(( $CEPH_RGW_PORT + $rgw ))
-	if [ "$new" -eq 1 ]; then
-	    if [ $overwrite_conf -eq 1 ]; then
-		    dnsname=`hostname -f`
-		    cat <<EOF >> $conf
-[client.radosgw.rgw$rgw]
-        host = $HOSTNAME
-$DAEMONOPTS
-        keyring = $CEPH_OUT_DIR/keyring.client.radosgw.rgw$rgw
-        rgw socket path = $CEPH_OUT_DIR/sock.client.radosgw.rgw$rgw
-        rgw dns name = $dnsname
+if [ "$ec" -eq 1 ]; then
+    $SUDO $CEPH_ADM <<EOF
+osd erasure-code-profile set ec-profile m=2 k=1
+osd pool create ec 8 8 erasure ec-profile
+quit
 EOF
-		    mkdir -p $CEPH_OUT_DIR/htdocs
-		    mkdir -p $CEPH_OUT_DIR/fastcgi_sock
-		    APACHE2_MODULE_PATH="/usr/lib/apache2/modules"
-		    APACHE2_EXTRA_MODULES_NAME="mpm_prefork authz_core"
-		    for module in $APACHE2_EXTRA_MODULES_NAME
-		    do
-			    if [ -f "${APACHE2_MODULE_PATH}/mod_${module}.so" ]; then
-				    APACHE2_EXTRA_MODULES="${APACHE2_EXTRA_MODULES}LoadModule ${module}_module ${APACHE2_MODULE_PATH}/mod_${module}.so
-"
-			    fi 
-		    done
-		    echo $APACHE2_EXTRA_MODULES
-		    cat <<EOF > $CEPH_OUT_DIR/apache.conf
-LoadModule env_module /usr/lib/apache2/modules/mod_env.so
-LoadModule rewrite_module /usr/lib/apache2/modules/mod_rewrite.so
-LoadModule fastcgi_module /usr/lib/apache2/modules/mod_fastcgi.so
-$APACHE2_EXTRA_MODULES
-
-Listen $rgwport
-ServerName rgwtest.example.com
-
-ServerRoot $CEPH_OUT_DIR
-ErrorLog $CEPH_OUT_DIR/apache.error.log
-LogFormat "%h l %u %t \"%r\" %>s %b \"{Referer}i\" \"%{User-agent}i\"" combined
-CustomLog $CEPH_OUT_DIR/apache.access.log combined
-PidFile $CEPH_OUT_DIR/apache.pid
-DocumentRoot $CEPH_OUT_DIR/htdocs
-FastCgiIPCDir $CEPH_OUT_DIR/fastcgi_sock
-FastCgiExternalServer $CEPH_OUT_DIR/htdocs/rgw.fcgi -socket $CEPH_OUT_DIR/sock.client.radosgw.rgw$rgw
-RewriteEngine On
-
-RewriteRule ^/([a-zA-Z0-9-_.]*)([/]?.*) /rgw.fcgi?page=$1&params=$2&%{QUERY_STRING} [E=HTTP_AUTHORIZATION:%{HTTP:Authorization},L]
-
-# Set fastcgi environment variables.
-# Note that this is separate from Unix environment variables!
-SetEnv RGW_LOG_LEVEL 20
-SetEnv RGW_PRINT_CONTINUE yes
-SetEnv RGW_SHOULD_LOG yes
-
-<Directory $CEPH_OUT_DIR/htdocs>
-  Options +ExecCGI
-  AllowOverride All
-  SetHandler fastcgi-script
-</Directory>
-
-AllowEncodedSlashes On
-ServerSignature Off
-EOF
-		    $SUDO $CEPH_ADM auth get-or-create client.radosgw.rgw$rgw osd 'allow rwx' mon 'allow r' -o $CEPH_OUT_DIR/keyring.client.radosgw.rgw$rgw
-
-		    #akey=`echo $$ | md5sum | cut -c 1-20`
-		    #skey=`dd if=/dev/urandom of=/tmp/random.$$ bs=1 count=40 2>/dev/null ; base64 < /tmp/random.$$ ; rm /tmp/random.$$`
-		    akey='0555b35654ad1656d804'
-		    skey='h7GhxuBLTrlhVUyxSPUKUV8r/2EI4ngqJxD7iBdBYLhwluN30JaT3Q=='
-		    echo access key $akey
-		    echo secret key $skey
-		    $CEPH_BIN/radosgw-admin user create --uid tester --access-key $akey --secret $skey --display-name 'M. Tester' --email tester@ceph.com -c $conf
-	    fi
-	fi
-	echo start rgw$rgw on http://localhost:$rgwport
-	run 'rgw' $SUDO $CEPH_BIN/radosgw -n client.radosgw.rgw$rgw $ARGS
-	run 'apache2' $SUDO apache2 -f $CEPH_OUT_DIR/apache.conf
-    done
 fi
 
-echo "started.  stop.sh to stop.  see out/* (e.g. 'tail -f out/????') for debug output."
-
 do_cache() {
     while [ -n "$*" ]; do
 	p="$1"
@@ -647,7 +609,53 @@ EOF
 }
 do_hitsets $hitset
 
+do_rgw()
+{
+    # Start server
+    echo start rgw on http://localhost:$CEPH_RGW_PORT
+    RGWDEBUG=""
+    if [ "$debug" -ne 0 ]; then
+        RGWDEBUG="--debug-rgw=20"
+    fi
+    $CEPH_BIN/radosgw --log-file=${CEPH_OUT_DIR}/rgw.log ${RGWDEBUG} --debug-ms=1
+
+    # Create S3 user
+    local akey='0555b35654ad1656d804'
+    local skey='h7GhxuBLTrlhVUyxSPUKUV8r/2EI4ngqJxD7iBdBYLhwluN30JaT3Q=='
+    echo "setting up user testid"
+    $CEPH_BIN/radosgw-admin user create --uid testid --access-key $akey --secret $skey --display-name 'M. Tester' --email tester@ceph.com -c $conf_fn > /dev/null
+
+    # Create S3-test users
+    # See: https://github.com/ceph/s3-tests
+    echo "setting up s3-test users"
+    $CEPH_BIN/radosgw-admin user create \
+        --uid 0123456789abcdef0123456789abcdef0123456789abcdef0123456789abcdef \
+        --access-key ABCDEFGHIJKLMNOPQRST \
+        --secret abcdefghijklmnopqrstuvwxyzabcdefghijklmn \
+        --display-name youruseridhere \
+        --email s3@example.com -c $conf_fn > /dev/null
+    $CEPH_BIN/radosgw-admin user create \
+        --uid 56789abcdef0123456789abcdef0123456789abcdef0123456789abcdef01234 \
+        --access-key NOPQRSTUVWXYZABCDEFG \
+        --secret nopqrstuvwxyzabcdefghijklmnabcdefghijklm \
+        --display-name john.doe \
+        --email john.doe@example.com -c $conf_fn > /dev/null
+
+    # Create Swift user
+    echo "setting up user tester"
+    $CEPH_BIN/radosgw-admin user create --subuser=tester:testing --display-name=Tester-Subuser --key-type=swift --secret=asdf > /dev/null
+}
+if [ "$start_rgw" -eq 1 ]; then
+    do_rgw
+fi
+
+echo "started.  stop.sh to stop.  see out/* (e.g. 'tail -f out/????') for debug output."
+
 echo ""
 echo "export PYTHONPATH=./pybind"
 echo "export LD_LIBRARY_PATH=.libs"
 
+if [ "$CEPH_DIR" != "$PWD" ]; then
+    echo "export CEPH_CONF=$conf_fn"
+    echo "export CEPH_KEYRING=$keyring_fn"
+fi
